from django.db import models
from django.contrib.postgres.fields import JSONField, ArrayField
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
# from mptt.models import MPTTModel, TreeForeignKey
# from django.urls import reverse, NoReverseMatch
import uuid


class EmailTemplate(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(_('Name'), max_length=255, unique=True)

    subject = models.CharField(_('Subject'), max_length=255)
    mjml_template = models.TextField(_('MJML template'), default='', blank=True)
    html_template = models.TextField(_('HTML template'), default='', blank=True)
    text_template = models.TextField(_('Text template'), default='', blank=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Email template')
        verbose_name_plural = _('Email templates')
        ordering = ('name',)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        from .utils import mjml_to_html
        from .html2text import html2text
        self.html_template = mjml_to_html(self.mjml_template)
        self.text_template = html2text(self.html_template)
        super(EmailTemplate, self).save(*args, **kwargs)

    def _template_header(self):
        CODE = '{% load i18n l10n pcart_core staticfiles %}\n'
        return CODE

    def _render_template(self, template, context={}):
        from django.template import Context, Template
        template = self._template_header() + template
        t = Template(template)
        c = Context(context)
        return t.render(c)

    def render_subject(self, context={}):
        return self._render_template(self.subject, context).replace('\n', '')

    def render_html(self, context={}):
        return self._render_template(self.html_template, context)

    def render_text(self, context={}):
        return self._render_template(self.text_template, context)


class EmailMessage(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    site = models.ForeignKey(
        Site, verbose_name=_('Site'), related_name='email_messages', on_delete=models.PROTECT)

    template = models.ForeignKey(
        EmailTemplate, verbose_name=_('Template'), related_name='email_messages', on_delete=models.PROTECT)

    data = JSONField(_('Data'), default=dict, blank=True)

    sent = models.BooleanField(_('Sent'), default=False)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Email message')
        verbose_name_plural = _('Email messages')
        ordering = ('-added',)

    def __str__(self):
        return self.subject()

    def subject(self):
        return self.data.get('subject').replace('\n', '')

    def recipients(self):
        return self.data.get('recipients')

    def sender(self):
        return self.data.get('sender')

    def html_body(self):
        return self.data.get('html')

    def text_body(self):
        return self.data.get('text')

    def send(self):
        from .tasks import send_email
        send_email.delay(self.id)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-27 14:48
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailMessage',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Data')),
                ('sent', models.BooleanField(default=False, verbose_name='Sent')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='email_messages', to='sites.Site', verbose_name='Site')),
            ],
            options={
                'ordering': ('-added',),
                'verbose_name': 'Email message',
                'verbose_name_plural': 'Email messages',
            },
        ),
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('mjml_template', models.TextField(blank=True, default='', verbose_name='MJML template')),
                ('html_template', models.TextField(blank=True, default='', verbose_name='HTML template')),
                ('text_template', models.TextField(blank=True, default='', verbose_name='Text template')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Email template',
                'verbose_name_plural': 'Email templates',
            },
        ),
        migrations.AddField(
            model_name='emailmessage',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='email_messages', to='pcart_messaging.EmailTemplate', verbose_name='Template'),
        ),
    ]



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.html import format_html
from django.utils.text import mark_safe
from django.urls import reverse
from .models import EmailTemplate, EmailMessage


class EmailTemplateAdmin(admin.ModelAdmin):
    list_display = ('name', 'subject')
    fields = ('name', 'subject', 'mjml_template', 'template_preview')
    readonly_fields = ('template_preview',)

    def template_preview(self, obj):
        TEMPLATE = '''
<div style="border:1px solid #ccc;">{html}</div>
<pre>
{text}
</pre>
'''
        context = {
            'html': obj.html_template,
            'text': obj.text_template,
        }
        result = mark_safe(TEMPLATE.format(**context))
        return result


admin.site.register(EmailTemplate, EmailTemplateAdmin)


class EmailMessageAdmin(admin.ModelAdmin):
    list_display = ('subject', 'site', 'recipients', 'sender', 'sent', 'added')
    readonly_fields = ('message_preview',)
    fields = ('site', 'template', 'sent', 'message_preview')
    actions = ['send_message']

    def send_message(self, request, queryset):
        for obj in queryset:
            obj.send()
    send_message.short_description = _('Send message')

    def message_preview(self, obj):
        TEMPLATE = '''
<strong>{To}:</strong> {to}<br>
<strong>{From}:</strong> {from}<br>
<strong>{Subject}:</strong> {subject}<br>
<div style="border:1px solid #ccc;">{html}</div>
<pre>
{text}
</pre>
'''
        context = {
            'To': _('To'),
            'From': _('From'),
            'Subject': _('Subject'),
            'to': ', '.join(obj.recipients()),
            'from': obj.sender(),
            'subject': obj.subject(),
            'html': obj.html_body(),
            'text': obj.text_body(),
        }
        result = mark_safe(TEMPLATE.format(**context))
        return result

admin.site.register(EmailMessage, EmailMessageAdmin)




class PCartBaseEmailBackend:
    def __init__(self, message_obj):
        self.message_obj = message_obj

    def send(self):
        pass


class PCartSimpleEmailBackend(PCartBaseEmailBackend):
    def send(self):
        from django.core.mail import EmailMultiAlternatives
        mail = EmailMultiAlternatives(
            subject=self.message_obj.subject(),
            body=self.message_obj.text_body(),
            from_email=self.message_obj.sender(),
            to=self.message_obj.recipients(),
        )
        mail.attach_alternative(self.message_obj.html_body(), 'text/html')

        try:
            mail.send()
            self.message_obj.sent = True
            self.message_obj.save()
        except Exception as e:
            self.message_obj.data['error'] = str(e)
            self.message_obj.save()



import subprocess
from django.conf import settings


def mjml_to_html(mjml_code):
    cmd_args = getattr(settings, 'PCART_MJML_COMMAND', 'mjml')
    if not isinstance(cmd_args, list):
        cmd_args = [cmd_args]
        cmd_args.extend(['-i', '-s'])

    try:
        p = subprocess.Popen(cmd_args, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        html = p.communicate(mjml_code.encode('utf8'))[0].decode('utf8')
    except (IOError, OSError) as e:
        raise RuntimeError(
            'Problem to run command "{}"\n'.format(' '.join(cmd_args)) +
            '{}\n'.format(e) +
            'Check that mjml is installed and allow permissions for execute.\n' +
            'See https://github.com/mjmlio/mjml#installation'
        )
    return html


def create_email_message(template_name, recipients, sender, context={}, request=None):
    from .models import EmailMessage, EmailTemplate
    from django.contrib.sites.shortcuts import get_current_site

    if request is not None:
        site = get_current_site(request)
    else:
        from django.contrib.sites.models import Site
        site = Site.objects.get_current()

    try:
        template = EmailTemplate.objects.get(name=template_name)

        data = {
            'recipients': recipients,
            'sender': sender,
            'subject': template.render_subject(context),
            'html': template.render_html(context),
            'text': template.render_text(context),
        }

        message = EmailMessage(
            site=site,
            template=template,
            data=data,
        )
        message.save()
        return message
    except EmailTemplate.DoesNotExist:
        pass



from celery import task
from django.utils.module_loading import import_string
from django.conf import settings


@task
def send_email(obj_id):
    from .models import EmailMessage
    message = EmailMessage.objects.get(id=obj_id)
    backend_class = import_string(settings.PCART_EMAIL_BACKEND)
    backend = backend_class(message)
    backend.send()
